﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace BinaryTreeUI
{
    public class AVLTree:Tree<AvlTreeNode>
    {
        /// <summary>
        /// 深度
        /// </summary>
        public int Depth {get;private set ;}

        #region Insert

        protected override void Insert(AvlTreeNode node)
        {
            if (Root == null)
            {
                Root = node;
                return;
            }

            FindPosition(Root, node);

            var parent = node.Parent;
            if (node == parent.LeftChild)
                parent.Balance++;
            else parent.Balance--;

            if (parent.Balance != 0) AdjustInsert(parent);
        }

        private void AdjustInsert(AvlTreeNode node)
        {
            if (node.Balance > 1)
            {
                BalanceRightRotate(node);
            }
            else if (node.Balance < -1)
            {
                BalanceLeftRotate(node);
            }
            else
            {
                var root = node.Parent;
                if (root != null)
                {
                    if (node == root.LeftChild)
                        root.Balance++;
                    else root.Balance--;

                    AdjustInsert(root);
                }
            }
        }
        #endregion

        #region Remove

        protected override void Remove(AvlTreeNode node)
        {
            if (node == Root)
            {
                Root = null;
                return;
            }

            var parent = node.Parent;
            if (node == parent.LeftChild)
            {
                parent.LeftChild = null;
                parent.Balance--;
            }
            else
            {
                parent.RightChild = null;
                parent.Balance++;
            }

            if (parent.Balance != 1 || parent.Balance != -1) AdjustRemove(parent);
        }

        private void AdjustRemove(AvlTreeNode node)
        {
            if (node.Balance > 1)
            {
                node = BalanceRightRotate(node);
            }
            else if (node.Balance < -1)
            {
                node = BalanceLeftRotate(node);
            }

            var parent = node.Parent;
            if (node.Balance == 0 && parent != null)
            {
                if (node == parent.LeftChild)
                    parent.Balance--;
                else parent.Balance++;

                if (parent.Balance != 1 && parent.Balance != -1) AdjustRemove(parent);
            }

        }

        /// <summary>
        /// 左旋转（可能包含先右旋），重新设置平衡度。
        /// </summary>
        /// <param name="node">旋转的轴节点</param>
        /// <returns>新的轴节点（原轴的子节点或子子节点）</returns>
        private AvlTreeNode BalanceLeftRotate(AvlTreeNode node)
        {
            var child = node.RightChild;

            if (child.Balance > 0)
            {
                var grand = child.LeftChild;
                RightRotate(grand, child);

                if (grand.Balance < 0)
                {
                    grand.Balance = -2;
                    child.Balance = 0;
                }
                else
                {
                    var temp = grand.Balance;
                    grand.Balance = -1 * child.Balance;
                    child.Balance = -1 * temp;
                }
                child = grand;
            }
            LeftRotate(child, node);
            node.Balance = -1 * child.Balance - 1;
            child.Balance = child.Balance < -1 ? 0 : child.Balance + 1;
            return child;
        }

        private AvlTreeNode BalanceRightRotate(AvlTreeNode node)
        {
            var child = node.LeftChild;
            if (child.Balance < 0)
            {
                var grand = child.RightChild;
                LeftRotate(grand, child);

                if (grand.Balance > 0)
                {
                    grand.Balance = 2;
                    child.Balance = 0;
                }
                else
                {
                    var temp = grand.Balance;
                    grand.Balance = -1 * child.Balance;
                    child.Balance = -1 * temp;
                }
                child = grand;
            }
            RightRotate(child, node);

            node.Balance = -1 * child.Balance + 1;
            child.Balance = child.Balance > 1 ? 0 : child.Balance - 1;
            return child;
        }
        #endregion
    }
}
