﻿using System.Windows;
using System.Windows.Controls;
using NodePainter = BinaryTreeUI.NodePainter<BinaryTreeUI.AvlTreeNode>;

namespace BinaryTreeUI
{
    /// <summary>
    /// Interaction logic for DemoAVL.xaml
    /// </summary>
    public partial class DemoAVL : Page
    {
        AVLTree tree = new AVLTree();
        public DemoAVL()
        {
            InitializeComponent();
            this.ShowsNavigationUI = false;
        }

        private void btnAdd_Click(object sender, RoutedEventArgs e)
        {
            int value;
            if (!int.TryParse(txtValue.Text, out value))
            {
                MessageBox.Show("Please enter a valid integer.");
                return;
            }

            tree.Insert(value);
            PaintTree();

            txtValue.Clear();
            txtValue.Focus();
        }

        private void PaintTree()
        {
            canvas1.Children.Clear();
            var painter = new NodePainter();
            painter.Paint(tree.Root, canvas1);
        }


        private void btnRemove_Click(object sender, RoutedEventArgs e)
        {
            int value;
            if (!int.TryParse(txtValue.Text, out value))
            {
                MessageBox.Show("Please enter a valid integer.");
                return;
            }

            var success = this.tree.Remove(value);

            tbCue.Text = success ? (value + " is removed succesfully") : (value + " is not Found");
            if (success)
            {
                PaintTree();
            }
        }

        private void btnShow_Click(object sender, RoutedEventArgs e)
        {
            var array = 
                new int[] { 8, 18, 28, 38, 20, 55, 66, 50, 49, 48, 47, 46, 45, 44, 43, 42, 41 };
                //new int[] { 8, 4,9,3,6,12,5};

            this.tree = new AVLTree();
            foreach (var item in array)
            {
                this.tree.Insert(item);
            }
            canvas1.Children.Clear();
            var painter = new NodePainter();
            painter.Paint(tree.Root, canvas1);
        }
    }
}
